==> bibledit-1.4.tar.gz: bibledit-1.4/src/clean_usfm.cpp <==
/*
** Copyright (C) 2003-2005 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "utilities.h"
#include "clean_usfm.h"
#include "directories.h"
#include "gwrappers.h"


CleanUsfm::CleanUsfm (vector <ustring>& input)
// Clean the lines provided: each main USFM starts a new line.
{
  // Let scripturechecks clean the lines up.
  ustring inputfile = gw_build_filename (directories_get_temp(), "clean_usfm.input");
  ustring outputfile  = gw_build_filename (directories_get_temp(), "clean_usfm.output");
  write_lines (inputfile, input);
  ustring command = "sc-input-usfm '" + inputfile + "' > '" + outputfile + "'";
  system (command.c_str());
  // Error and parse variables.
  GError *gerror = NULL;
  GMarkupParseContext *context = NULL;
  try {
    // Read the xml file.
    gchar *contents;
    gsize  length;
    if (!g_file_get_contents (outputfile.c_str(), &contents, &length, &gerror)) {
      throw runtime_error (gerror->message);
    }
    // If length is (about) zero, don't parse it, because it does not have data,
    // and parsing an empty file gives a segmentation error.
    if (length < 10)
      throw runtime_error ("File too short");  
    // Set up parser.
    GMarkupParser parser = {
      start_element_handler,
      NULL,
      text_handler,
      NULL,
      error_handler
    };
    // Parse xml file.
    context = g_markup_parse_context_new (&parser, GMarkupParseFlags (0), gpointer (this), NULL);
    if (!g_markup_parse_context_parse (context, contents, length, NULL)) {
      throw runtime_error (gerror->message);
    }
    if (!g_markup_parse_context_end_parse (context, NULL)) {
      throw runtime_error (gerror->message);
    }
    g_markup_parse_context_free (context);
  }
  catch (exception & ex) {
    gw_critical (ex.what ());
    if (gerror)
      g_error_free (gerror);
    if (context)
      g_markup_parse_context_free (context);
  }
  // Clean temporary files away.
  unlink (inputfile.c_str());
  unlink (outputfile.c_str());
}


CleanUsfm::~CleanUsfm ()
{
}


void CleanUsfm::start_element_handler (GMarkupParseContext *context,
                                       const gchar         *element_name,
                                       const gchar        **attribute_names,
                                       const gchar        **attribute_values,
                                       gpointer             user_data,
                                       GError             **error)
{
  ((CleanUsfm *) user_data)->start_element_handler (element_name, attribute_values);
}


void CleanUsfm::text_handler (GMarkupParseContext *context,
                              const gchar         *text,
                              gsize                text_len,
                              gpointer             user_data,
                              GError             **error)
{
  ((CleanUsfm *) user_data)->text_handler (text);
}


void CleanUsfm::error_handler (GMarkupParseContext *context,
                               GError              *error,
                               gpointer             user_data)
{
  gw_critical (error->message);
}


void CleanUsfm::start_element_handler (const gchar  *element_name,
                                       const gchar **attribute_values)
{
}


void CleanUsfm::text_handler (const gchar *text)
{
  ParseLine parseline (text);
  for (unsigned int i = 0; i < parseline.lines.size(); i++)
    lines.push_back (parseline.lines[i]);
}

==> bibledit-1.5.tar.gz: bibledit-1.5/src/clean_usfm.cpp <==
/*
** Copyright (C) 2003-2005 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "utilities.h"
#include "clean_usfm.h"
#include "directories.h"
#include "gwrappers.h"


CleanUsfm::CleanUsfm (vector <ustring>& input)
// Clean the lines provided: each main USFM starts a new line.
{
  // Let scripturechecks clean the lines up.
  ustring inputfile = gw_build_filename (directories_get_temp(), "clean_usfm.input");
  ustring outputfile  = gw_build_filename (directories_get_temp(), "clean_usfm.output");
  write_lines (inputfile, input);
  ustring command = "sc-input-usfm '" + inputfile + "' > '" + outputfile + "'";
  system (command.c_str());
  // Error and parse variables.
  GError *gerror = NULL;
  GMarkupParseContext *context = NULL;
  try {
    // Read the xml file.
    gchar *contents;
    gsize  length;
    if (!g_file_get_contents (outputfile.c_str(), &contents, &length, &gerror)) {
      throw runtime_error (gerror->message);
    }
    // If length is (about) zero, don't parse it, because it does not have data,
    // and parsing an empty file gives a segmentation error.
    if (length < 10)
      throw runtime_error ("File too short");  
    // Set up parser.
    GMarkupParser parser = {
      start_element_handler,
      NULL,
      text_handler,
      NULL,
      error_handler
    };
    // Parse xml file.
    context = g_markup_parse_context_new (&parser, GMarkupParseFlags (0), gpointer (this), NULL);
    if (!g_markup_parse_context_parse (context, contents, length, NULL)) {
      throw runtime_error (gerror->message);
    }
    if (!g_markup_parse_context_end_parse (context, NULL)) {
      throw runtime_error (gerror->message);
    }
    g_markup_parse_context_free (context);
  }
  catch (exception & ex) {
    gw_critical (ex.what ());
    if (gerror)
      g_error_free (gerror);
    if (context)
      g_markup_parse_context_free (context);
  }
  // Clean temporary files away.
  unlink (inputfile.c_str());
  unlink (outputfile.c_str());
}


CleanUsfm::~CleanUsfm ()
{
}


void CleanUsfm::start_element_handler (GMarkupParseContext *context,
                                       const gchar         *element_name,
                                       const gchar        **attribute_names,
                                       const gchar        **attribute_values,
                                       gpointer             user_data,
                                       GError             **error)
{
  ((CleanUsfm *) user_data)->start_element_handler (element_name, attribute_values);
}


void CleanUsfm::text_handler (GMarkupParseContext *context,
                              const gchar         *text,
                              gsize                text_len,
                              gpointer             user_data,
                              GError             **error)
{
  ((CleanUsfm *) user_data)->text_handler (text);
}


void CleanUsfm::error_handler (GMarkupParseContext *context,
                               GError              *error,
                               gpointer             user_data)
{
  gw_critical (error->message);
}


void CleanUsfm::start_element_handler (const gchar  *element_name,
                                       const gchar **attribute_values)
{
}


void CleanUsfm::text_handler (const gchar *text)
{
  ParseLine parseline (text);
  for (unsigned int i = 0; i < parseline.lines.size(); i++) {
    lines.push_back (parseline.lines[i]);
  }
}
